@@ -1,10 +1,8 @@
 package Jifty::DBI;
-use warnings; 
+use warnings;
 use strict;
 
-
-our $VERSION = '0.05_02';
-
+$Jifty::DBI::VERSION = '0.39_99';
 
 =head1 NAME
 
@@ -15,17 +13,17 @@ Jifty::DBI - An object-relational persistence framework
 Jifty::DBI deals with databases, so that you don't have to.
 
 This module provides an object-oriented mechanism for retrieving and
-updating data in a DBI-accesible database. 
+updating data in a DBI-accessible database. 
 
-This module is the direct descendent of DBIx::SearchBuilder. If you're familiar
+This module is the direct descendent of L<DBIx::SearchBuilder>. If you're familiar
 with SearchBuilder, Jifty::DBI should be quite familiar to you.
 
 =head2 What is it trying to do. 
 
 Jifty::DBI::Record abstracts the agony of writing the common and generally 
-simple SQL statements needed to serialize and De-serialize an object to the
+simple SQL statements needed to serialize and de-serialize an object to the
 database.  In a traditional system, you would define various methods on 
-your object 'create', 'find', 'modify', and 'delete' being the most common. 
+your object 'create', 'read', 'update', and 'delete' being the most common. 
 In each method you would have a SQL statement like: 
 
   select * from table where value='blah';
@@ -40,10 +38,10 @@ basically the same.
 
 Enter, Jifty::DBI::Record. 
 
-With::Record, you can in the simple case, remove all of that code and 
-replace it by defining two methods and inheriting some code.  Its pretty 
-simple, and incredibly powerful.  For more complex cases, you can, gasp, 
-do more complicated things by overriding certain methods.  Lets stick with
+With ::Record, you can in the simple case, remove all of that code and 
+replace it by defining two methods and inheriting some code.  It's pretty 
+simple and incredibly powerful.  For more complex cases, you can 
+do more complicated things by overriding certain methods.  Let's stick with
 the simple case for now. 
 
 
@@ -86,22 +84,20 @@ The table looks like so:
 
 First, let's define our record class in a new module named "Simple.pm".
 
-
   use warnings;
   use strict;
-   
-  package Simple::Schema;
+
+  package Simple;
   use Jifty::DBI::Schema;
-  
-  column foo => type is 'text';
-  column bar => type is 'text';
-  
-  package Simple; 
-  use base qw(Jifty::DBI::Record);
+  use Jifty::DBI::Record schema {
+    column foo => type is 'text';
+    column bar => type is 'text';
+  };
 
   # your custom code goes here.
 
   1;
+
 Like all perl modules, this needs to end with a true value. 
 
 Now, on to the code that will actually *do* something with this object. 
@@ -113,24 +109,24 @@ This code would be placed in your Perl script.
 Use two packages, the first is where I get the DB handle from, the latter 
 is the object I just created. 
 
-  
+
   my $handle = Jifty::DBI::Handle->new();
-  $handle->Connect(
-      Driver   => 'Pg',
-      Database => 'test',
-      Host     => 'reason',
-      User     => 'mhat',
-      Password => ''
+  $handle->connect(
+      driver   => 'Pg',
+      database => 'test',
+      host     => 'reason',
+      user     => 'mhat',
+      password => ''
   );
-  
+
 Creates a new Jifty::DBI::Handle, and then connects to the database using 
 that handle.  Pretty straight forward, the password '' is what I use 
 when there is no password.  I could probably leave it blank, but I find 
 it to be more clear to define it.
 
- 
- my $s = Simple->new($handle);
- 
+
+ my $s = Simple->new( handle => $handle );
+
  $s->load_by_cols(id=>1); 
 
 
@@ -138,7 +134,7 @@ it to be more clear to define it.
 
 =item load_by_cols
 
-Takes a hash of column =>value pairs and returns the *first* to match. 
+Takes a hash of column => value pairs and returns the *first* to match. 
 First is probably lossy across databases vendors. 
 
 =item load_from_hash
@@ -184,20 +180,20 @@ Currently Set<Field> updates the data in the database as soon as you call
 it.  In the future I hope to extend ::Record to better support transactional
 operations, such that updates will only happen when "you" say so.
 
-Finally, adding a removing records from the database.  ::Record provides a 
-Create method which simply takes a hash of key=>value pairs.  The keys 
-exactly	map to database columns. 
+Finally, adding and removing records from the database.  ::Record provides a 
+Create method which simply takes a hash of key => value pairs.  The keys 
+exactly map to database columns. 
 
  ## Get a new record object.
- $s1 = Simple->new($handle);
+ $s1 = Simple->new( handle => $handle );
  my ($id, $status_msg) = $s1->create(id  => 4,
- 	           foo => 'Foooooo', 
- 	           bar => 'Barrrrr');
+                   foo => 'Foooooo', 
+                   bar => 'Barrrrr');
 
 Poof! A new row in the database has been created!  Now lets delete the 
 object! 
 
- my $s2 = Simple->new($handle);
+ my $s2 = Simple->new( handle => $handle );
  $s2->load_by_cols(id=>4);
  $s2->delete();
 
